@@ -89,7 +89,7 @@ func readStateScripts(tr *tar.Reader, header *tar.Header, cb ScriptsReadFn) erro
 
 	for {
 		hdr, err := getNext(tr)
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			return nil
 		} else if err != nil {
 			return errors.Wrapf(err,
@@ -165,7 +165,7 @@ func (ar *Reader) readHeader(headerSum []byte, comp artifact.Compressor) error {
 func (ar *Reader) populateArtifactInfo(version int, tr *tar.Reader) error {
 	var hInfo artifact.HeaderInfoer
 	switch version {
-	case 1, 2:
+	case 2:
 		hInfo = new(artifact.HeaderInfo)
 	case 3:
 		hInfo = new(artifact.HeaderInfoV3)
@@ -252,30 +252,6 @@ func (ar *Reader) GetHandlers() map[int]handlers.Installer {
 	return ar.installers
 }
 
-func (ar *Reader) readHeaderV1() error {
-	if ar.shouldBeSigned {
-		return errors.New("reader: expecting signed artifact; " +
-			"v1 is not supporting signatures")
-	}
-	hdr, err := getNext(ar.menderTarReader)
-	if err != nil {
-		return errors.New("reader: error reading header")
-	}
-	if !strings.HasPrefix(hdr.Name, "header.tar") {
-		return errors.Errorf("reader: invalid header element: %v", hdr.Name)
-	}
-
-	comp, err := artifact.NewCompressorFromFileName(hdr.Name)
-	if err != nil {
-		return errors.New("reader: can't get compressor")
-	}
-
-	if err = ar.readHeader(nil, comp); err != nil {
-		return err
-	}
-	return nil
-}
-
 func (ar *Reader) readManifest(name string) error {
 	buf := bytes.NewBuffer(nil)
 	if err := readNext(ar.menderTarReader, buf, name); err != nil {
@@ -370,7 +346,7 @@ func (ar *Reader) readHeaderV3(version []byte) error {
 
 	for {
 		hdr, err := ar.menderTarReader.Next()
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			return errors.New("The artifact does not contain all required fields")
 		}
 		if err != nil {
@@ -587,7 +563,7 @@ func (ar *Reader) ReadArtifactHeaders() error {
 
 	switch ver.Version {
 	case 1:
-		err = ar.readHeaderV1()
+		err = errors.New("reader: Mender-Artifact version 1 is no longer supported")
 	case 2:
 		err = ar.readHeaderV2(vRaw)
 	case 3:
@@ -599,18 +575,16 @@ func (ar *Reader) ReadArtifactHeaders() error {
 		return err
 	}
 
-	for no, us := range ar.updateStorers {
-		err = us.Initialize(ar.hInfo, ar.augmentedhInfo, ar.installers[no])
-		if err != nil {
-			return err
-		}
-	}
-
 	return nil
 }
 
 func (ar *Reader) ReadArtifactData() error {
-	err := ar.readData(ar.menderTarReader)
+	err := ar.initializeUpdateStorers()
+	if err != nil {
+		return err
+	}
+
+	err = ar.readData(ar.menderTarReader)
 	if err != nil {
 		return err
 	}
@@ -689,13 +663,35 @@ func (ar *Reader) setInstallers(upd []artifact.UpdateType, augmented bool) error
 				return err
 			}
 		}
+	}
+	return nil
+}
 
+func (ar *Reader) initializeUpdateStorers() error {
+	if len(ar.updateStorers) == len(ar.installers) {
+		// Already done.
+		return nil
+	}
+
+	for i, update := range ar.installers {
 		var err error
-		ar.updateStorers[i], err = ar.installers[i].NewUpdateStorer(update.Type, i)
+
+		ar.updateStorers[i], err = ar.installers[i].NewUpdateStorer(update.GetUpdateType(), i)
+		if err != nil {
+			return err
+		}
+
+		err = ar.updateStorers[i].Initialize(ar.hInfo, ar.augmentedhInfo, ar.installers[i])
 		if err != nil {
 			return err
 		}
 	}
+	if len(ar.updateStorers) != len(ar.installers) {
+		return errors.Errorf(
+			"Mismatch between installers/updateStorers lengths (%d != %d). Should not happen!",
+			len(ar.updateStorers), len(ar.installers))
+	}
+
 	return nil
 }
 
@@ -820,7 +816,7 @@ func (ar *Reader) readHeaderUpdate(tr *tar.Reader, hdr *tar.Header, augmented bo
 
 		var err error
 		hdr, err = getNext(tr)
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			return nil
 		} else if err != nil {
 			return errors.Wrapf(err,
@@ -831,7 +827,7 @@ func (ar *Reader) readHeaderUpdate(tr *tar.Reader, hdr *tar.Header, augmented bo
 
 func (ar *Reader) readNextDataFile(tr *tar.Reader) error {
 	hdr, err := getNext(tr)
-	if err == io.EOF {
+	if errors.Cause(err) == io.EOF {
 		return io.EOF
 	} else if err != nil {
 		return errors.Wrapf(err, "reader: error reading Payload file: [%v]", hdr)
@@ -858,7 +854,7 @@ func (ar *Reader) readNextDataFile(tr *tar.Reader) error {
 func (ar *Reader) readData(tr *tar.Reader) error {
 	for {
 		err := ar.readNextDataFile(tr)
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			break
 		} else if err != nil {
 			return err
@@ -875,17 +871,39 @@ func readNext(tr *tar.Reader, w io.Writer, elem string) error {
 	if err != nil {
 		return errors.Wrap(err, "readNext: Failed to get next header")
 	}
-	if strings.HasPrefix(hdr.Name, elem) {
-		_, err := io.Copy(w, tr)
-		return errors.Wrap(err, "readNext: Failed to copy from tarReader to the writer")
+
+	if !strings.HasPrefix(hdr.Name, elem) {
+		return os.ErrInvalid
 	}
-	return os.ErrInvalid
+
+	buf := new(bytes.Buffer)
+	_, err = io.Copy(buf, tr)
+
+	// io.Copy() is not supposed to return EOF, but it can if the EOF is a
+	// wrapped error, which can happen if the underlying Reader is a network
+	// stream.
+	if err != nil && errors.Cause(err) != io.EOF {
+		return errors.Wrap(err, "readNext: Failed to copy from tarReader to the buffer")
+	}
+
+	// The reason we did not write directly into w above is that if the
+	// stream comes from a slow network socket, it may be sufficiently
+	// chopped up that the JSON cannot be parsed due to being partial, and
+	// the Write() call to the HeaderInfo struct does not like
+	// that. Therefore we collect everything in the buffer first, and then
+	// write it here in one go.
+	_, err = buf.WriteTo(w)
+	if err != nil {
+		return errors.Wrapf(err, "readNext: Could not parse header element %s", elem)
+	}
+
+	return nil
 }
 
 func getNext(tr *tar.Reader) (*tar.Header, error) {
 	for {
 		hdr, err := tr.Next()
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			// we've reached end of archive
 			return hdr, err
 		} else if err != nil {
@@ -941,7 +959,7 @@ func (ar *Reader) readAndInstallDataFiles(tar *tar.Reader, i handlers.Installer,
 
 	for {
 		hdr, err := tar.Next()
-		if err == io.EOF {
+		if errors.Cause(err) == io.EOF {
 			break
 		} else if err != nil {
 			return errors.Wrap(err, "Payload: error reading Artifact file header")
@@ -1000,12 +1018,17 @@ func (ar *Reader) readAndInstallDataFiles(tar *tar.Reader, i handlers.Installer,
 }
 
 func (ar *Reader) GetUpdateStorers() ([]handlers.UpdateStorer, error) {
+	err := ar.initializeUpdateStorers()
+	if err != nil {
+		return nil, err
+	}
+
 	length := len(ar.updateStorers)
 	list := make([]handlers.UpdateStorer, length)
 
 	for i := range ar.updateStorers {
 		if i >= length {
-			return []handlers.UpdateStorer{}, errors.New("Update payload numbers are not in strictly increasing numbers from zero")
+			return nil, errors.New("Update payload numbers are not in strictly increasing numbers from zero")
 		}
 		list[i] = ar.updateStorers[i]
 	}
