@@ -1,4 +1,4 @@
-// Copyright 2018 Northern.tech AS
+// Copyright 2019 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@ package artifact
 import (
 	"io"
 
-	"github.com/mendersoftware/go-liblzma"
+	"github.com/remyoudompheng/go-liblzma"
 )
 
 type CompressorLzma struct {
